
#include <stdio.h>
#include <string>
#include <iostream>
#include <boost/foreach.hpp>
#include <boost/filesystem.hpp>
#include <generated/config.h>

#include <connector/resourcemanager.h>
#include <connector/service/serviceprovider.h>
#include <connector/service/extension/player/tvplayer.h>
#include <connector/service/extension/player/dummyplayer.h>
#include <connector/service/extension/player/playerextension.h>
#include <connector/service/extension/application/applicationextension.h>
#include <connector/service/extension/application/application.h>
#include <connector/service/extension/application/controller/applicationcontroller.h>
#include <connector/service/extension/application/controller/spawnerimp.h>
#include <connector/service/extension/application/factories/aitfactory.h>
#include <connector/service/extension/application/factories/profile/ginga-ncl/nclprofile.h>
#include <connector/provider/ts/data/fileprovider.h>
#include <connector/provider/ts/data/networkprovider.h>
#include <connector/provider/ts/tsprovider.h>

//#include <connector/service/extension/download/downloaderextension.h>

#include <unistd.h>


using namespace std;

namespace fs = boost::filesystem;

typedef std::vector<tuner::app::Application *> Applications;


class FakeAppCtrl : public tuner::app::ApplicationController {
public:
	FakeAppCtrl( tuner::ServiceManager *mgr )
		: tuner::app::ApplicationController( mgr )
	{
		_spawner = new tuner::app::SpawnerImp(2);	// TuDu: 1,2,n...???  maxParalel..???
	}
	virtual ~FakeAppCtrl( void ) {}

	void getApplications( Applications &apps, bool asUser=true ) {
		BOOST_FOREACH( tuner::app::Application *app, _apps ) {
			if ((asUser && app->visibility() == tuner::app::visibility::both) || (!asUser)) {
				apps.push_back( app );
			}
		}
	}

	//	Zapper virtual methods
	virtual void onApplicationAdded( tuner::app::Application *app ) {
		_apps.push_back( app );
	}

	virtual void onApplicationRemoved( tuner::app::Application *app ) {
		Applications::iterator it=std::find( _apps.begin(), _apps.end(), app );
		if (it != _apps.end()) {
			_apps.erase( it );
		}
	}

	// virtual void onApplicationChanged( tuner::app::Application *app ) {
	// 	printf( "[FakeAppCtrl] On application changed\n" );
	// }
	
	virtual void reserveKeys( const std::vector<util::key::type> &/*keys*/ )  {}
	virtual void resizeVideo( float /*x*/, float /*y*/, float /*w*/, float /*h*/ ) {}
	virtual tuner::app::Spawner *spawner() { return _spawner; }

protected:
	//	Create extension
	virtual void setupFactories( tuner::app::ApplicationExtension* ext ) {
	  
		tuner::app::AitFactory* appfactory = new tuner::app::AitFactory(ext);
		ext->addFactory(appfactory);

		tuner::app::NCLProfile* nclprofile = new tuner::app::NCLProfile(ext);
		appfactory->addProfile(nclprofile);
	  
	}

private:
	tuner::app::Spawner *_spawner;
	Applications _apps;
};




void printHelp() {
	cout << endl << "Usage: zapperk [OPTIONS]"  << endl;
	cout << endl << "OPTIONS are:" << endl;
	cout << "-h, --help                    Display this information." << endl;
	cout << "-v, --version                 Display version." << endl;
	cout << "-b, --build                   Display build information." << endl;
	
}

void printVersion() {
	printf("zapperk %s by KATO\n", ZAPPERK_VERSION_STR);
	printf("Based on Ginga 0.11.2 & dtv-connector\n");
	printf("Copyright 2002-2010 The TeleMidia PUC-Rio Team.\n");
	printf("Copyright 2010 LIFIA - Facultad de Informática - Univ. Nacional de La Plata \n");
}

void printBuildInfo(){
	printf("zapperk %s.%s\n", ZAPPERK_VERSION_STR, SVN_REVISION);
}


//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
int main(int argc, char *argv[]) {

	for (int i = 1; i < argc; i++) {
		if ((strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "--help") == 0)) {
			printHelp();
			return 0;

		} else if ((strcmp(argv[i], "-v") == 0) ||
			    (strcmp(argv[i], "--version")) == 0) {

			printVersion();
			return 0;
		} else if ((strcmp(argv[i], "-b") == 0) ||
			(strcmp(argv[i], "--build")) == 0) {
				printBuildInfo();
				return 0;
		}
	}


	printf("\n   --------------------------| zapperk %s - PID: %d |--------------------------\n",ZAPPERK_VERSION_STR, getpid());


	void*	tuner           = NULL;
	void*	demuxer         = NULL;
	
	// TuDu:
	// Exceptions y/o Control de errores...

	// TuDu:
	// Args. para opcion input mode...
//	File system provider...
	tuner = new tuner::FileProvider("/home/lifia/ginga/share/ginga/ts/");


//	Network provider...	
//	tuner = new tuner::NetworkProvider("127.0.0.1", 14000);
//	tuner = new tuner::NetworkProvider("localhost", 14000);

	// TuDu:
	// DVB device provider...

	
	demuxer = new tuner::TSProvider((tuner::TSDataProvider*) tuner);

	
	tuner::ResourceManager* resMgr = new tuner::ResourceManager( "/tmp/test", 100, 100, 4096 );		
	tuner::ServiceProvider* serviceProv = new tuner::ServiceProvider((tuner::TSProvider*) demuxer, resMgr);

	// App. Extension.
	FakeAppCtrl* ctrl = new FakeAppCtrl(serviceProv);
	ctrl->setup();
	serviceProv->addExtension(ctrl->extension());

	
//	TuDu:
//	Play de audio y video ppal...
//	Para enganchar con AV Player (gstreamer, VLC, Xine, ...) ??
//	Player Extension.
	tuner::DummyPlayer* dummy = new tuner::DummyPlayer();	// Falta implementar en connector...
	tuner::TvPlayer* player = new tuner::TvPlayer(dummy);
	tuner::PlayerExtension* playextension = new tuner::PlayerExtension(serviceProv);
	playextension->registerPlayer(tuner::service::type::digital_television, player);
	serviceProv->addExtension(playextension);
	
	((tuner::TSProvider*)demuxer)->start();

//	Input from file system...	
	//((tuner::TSProvider*)demuxer)->setNetwork("/home/lifia/ginga/share/ginga/ts/sample.ts");
	//((tuner::TSProvider*)demuxer)->setNetwork("/home/lifia/ginga/share/ginga/ts/ts_lfa_publico.ts");
	//((tuner::TSProvider*)demuxer)->setNetwork(DATA_PREFIX_PATH "/share/ginga/ts/ts_afarte.ts");
	// Este contiene una app. ncl...
	((tuner::TSProvider*)demuxer)->setNetwork("/home/lifia/ginga/share/ginga/ts/ts_editingcommandstest.ts");

//	Input from tcp...	
//	((tuner::TSProvider*)demuxer)->setNetwork("127.0.0.1:14000");
//	((tuner::TSProvider*)demuxer)->setNetwork("localhost:14000");

	serviceProv->start();

/*	
 * 	TuDu:
 * 	Add lib... p.ej DFB...??
 * 	wait tuning...  captura de eventos de user...
	if (fileExists(DATA_PREFIX_PATH "/share/ginga/tuner/tuning.png")) {
		tuningWindow = new DFBWindow(data->x, data->y, data->w, data->h);
		s = ImagePlayer::renderImage(DATA_PREFIX_PATH "/share/ginga/tuner/tuning.png");

		((IWindow*)tuningWindow)->setCaps(((IWindow*)tuningWindow)->getCap("ALPHACHANNEL"));

		((IWindow*)tuningWindow)->draw();
		((IWindow*)tuningWindow)->show();
		((IWindow*)tuningWindow)->lowerToBottom();
		((IWindow*)tuningWindow)->renderFrom(s);
		delete s;
		s = NULL;
	}
*/

	// TuDu:
	// Seleccion ==> srvID etc...
	
	serviceProv->startService(0xe760);	// srvID - ID servicio TV digital.
		

	// TuDu:
	// Reemplazar super wait... :) :)   etc.. etc... etc...
	getchar();
	
	cout << "[zapperk] Process finished - PID: " << getpid() << endl;
	return 0;
}
